using System;
using System.IO;
using System.Reflection;

namespace MSDNMagazine.TFSPlugins
{
	/// <summary>Utilities to help reading and writing embedded resources.</summary>
	/// <remarks>This is used to access the stylesheet, xslt, and images.</remarks>
	public sealed class ResourceWriter
	{
		/// <summary>
		/// Protect the contructor from beeing called because this is a class containing only static members
		/// </summary>
		private ResourceWriter(){}

		/// <summary>Writes all the embedded resources with the specified postfix to disk.</summary>
		/// <param name="thisAssembly">The assembly containing the embedded resources.</param>
		/// <param name="postfix">The postfix to search for.</param>
		/// <param name="directory">The directory to write the resources to.</param>
		public static void WriteEmbeddedResources(
			Assembly thisAssembly,
			string postfix,
			string directory)
		{
			if (!directory.EndsWith("\\"))
			{
				directory += "\\";
			}

			if (!Directory.Exists(directory))
			{
				Directory.CreateDirectory(directory);
			}

			string[] names = thisAssembly.GetManifestResourceNames();

			foreach (string name in names)
			{
				if (name.EndsWith(postfix))
				{
					int startPos = name.LastIndexOf("Resources");
					WriteEmbeddedResource(
						thisAssembly,
						name,
						directory,
						name.Substring(startPos + "Resources".Length + 1));
				}
			}
		}

		/// <summary>Writes an embedded resource to disk.</summary>
		/// <param name="thisAssembly">The assembly containing the embedded resource.</param>
		/// <param name="name">The name of the embedded resource.</param>
		/// <param name="directory">The directory to write the resource to.</param>
		/// <param name="filename">The filename of the resource on disk.</param>
		public static void WriteEmbeddedResource(
			Assembly thisAssembly,
			string name,
			string directory,
			string filename)
		{
			const int size = 512;
			byte[] buffer = new byte[size];
			int count = 0;

			Stream input = thisAssembly.GetManifestResourceStream(name);
			Stream output = File.Open(directory + filename, FileMode.Create);

			try
			{
				while ((count = input.Read(buffer, 0, size)) > 0)
				{
					output.Write(buffer, 0, count);
				}
			}
			finally
			{
				output.Close();
			}
		}
	}
}
