﻿#region Copyright © 2009 Marcel de Vries
/* This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in a
 *    product, an acknowledgment in the product documentation is requested, as
 *    shown here:
 * 
 *    Portions copyright © 2009 Marcel de Vries (http://blogs.infosupport.com/blogs/marcelv).
 * 
 * 2. No substantial portion of this source code may be redistributed without
 *    the express written permission of the copyright holders, where
 *    "substantial" is defined as enough code to be recognizably from this code.
 */
#endregion
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using System.Xml.Linq;


namespace MSDNMagazine.TFSPlugins
{
    public static class SubstConfigurationFile
    {

        private static ImageList _imageList = null;
        private static XElement _configXmlFile = null;
        private static string _substExplorerFolder;
        private static string _personalConfigurationFile;


        private const string CONFIGFILENAME = "SubstMappings.xml";
        private const string SUBSTMAPPINGROOTELEMENT = "substMappings";
        private const string TEAMPROJECTELEMENT = "TeamProject";
        private const string TEAMPROJECTNAMEATTRIBUTE = "name";
        private const string MAPPINGELEMENT = "Mapping";
        private const string MAPPINGVERSIONCONTROLATTRIBUTE = "VersionControlPath";
        private const string MAPPINGSUBSTDRIVELETTERATTRIBUTE = "SubstDriveLetter";
        private const string MAPPINGUSECURRENWORKSPACEATTRIBUTE = "UseCurrentWorkspace";
        private const string MAPPINGNAMEATTRIBUTE = "Name";


        #region Private methods

        /// <summary>
        /// Initialization of the configuration file. Ensures a folder in your local appdata
        /// and writes also images and template XML config file to disk.
        /// </summary>
        private static void initialize()
        {
            if (!File.Exists(PersonalConfigurationFile))
            {
                if (!Directory.Exists(SubstExplorerFolder))
                {
                    Directory.CreateDirectory(SubstExplorerFolder);
                }
                // now copy the files to the local directory
                ResourceWriter.WriteEmbeddedResources(Assembly.GetExecutingAssembly(), "png", SubstExplorerFolder);
                ResourceWriter.WriteEmbeddedResources(Assembly.GetExecutingAssembly(), "xml", SubstExplorerFolder);
            }

            _configXmlFile = XElement.Load(PersonalConfigurationFile);
        }

        private static string SubstExplorerFolder
        {
            get{
                 if(_substExplorerFolder == null)
                 {
                      string appDataFolder = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                      _substExplorerFolder = Path.Combine(appDataFolder, "MSDNMagazine.TFSPlugins");
                 }
                return _substExplorerFolder;
            }
        }

        private static string PersonalConfigurationFile 
        {
            get{
                if (_personalConfigurationFile == null)
                 {
                     _personalConfigurationFile = Path.Combine(SubstExplorerFolder, CONFIGFILENAME);
                 }
                 return _personalConfigurationFile;
            }
        }


        /// <summary>
        /// Property that uses lazy evaluation to initialize the configuration file.
        /// </summary>
        private static XElement ConfigXmlFile
        {
            get
            {
                if (_configXmlFile == null)
                {
                    SubstConfigurationFile.initialize();
                }
                return _configXmlFile;
            }
        }

        /// <summary>
        /// Adds a team project node to the config file
        /// </summary>
        /// <param name="projectName">Team  Project name</param>
        private static void AddTeamProject(string projectName)
        {
            
            // check if project already exists
            XElement project = FindTeamProjectNode(projectName);
            if (project == null)
            {
                // new tag <TeamProject name="projectName"/>
                project = new XElement(TEAMPROJECTELEMENT, new XAttribute(TEAMPROJECTNAMEATTRIBUTE, projectName));
                ConfigXmlFile.Add(project);
            }

        }
        /// <summary>
        /// Removes complete Team Project node based on project name
        /// </summary>
        /// <param name="projectName">Team Project name</param>
        private static void RemoveTeamProject(string projectName)
        {
            XElement project = FindTeamProjectNode(projectName);
            project.Remove();
            Save();
        }

        /// <summary>
        /// Finds a specific XElement node in the configuration file based on the team project name
        /// </summary>
        /// <param name="projectName">Name of the team project</param>
        /// <returns>XElement referencing the team project. Null if projec tnot found.</returns>
        private static XElement FindTeamProjectNode(string projectName)
        {

            XElement project = (from mapping in ConfigXmlFile.Elements(TEAMPROJECTELEMENT)
                                where mapping.Attribute(TEAMPROJECTNAMEATTRIBUTE) != null
                                && mapping.Attribute(TEAMPROJECTNAMEATTRIBUTE).Value == projectName
                                select mapping).FirstOrDefault();
            return project;
        }

        /// <summary>
        /// Saves any changes made to the configuration file to disk. 
        /// </summary>
        private static void Save()
        {
            ConfigXmlFile.Save(PersonalConfigurationFile);
        }
        #endregion

        #region public methods
        /// <summary>
        /// Adds a new mapping tho the configuration file. Changes are persisted to disk when function returns
        /// </summary>
        /// <param name="projectName">Name of the Team Project</param>
        /// <param name="mappingName">Name of the mapping</param>
        /// <param name="versionControlPath">Version control path used to determine location on disk based on workspace mappings for current user.</param>
        /// <param name="substDriveLetter">Drive letter used for the subst.</param>
        public static void AddMappingToProject(string projectName, string mappingName, string versionControlPath, string substDriveLetter)
        {
           
            XElement project = FindTeamProjectNode(projectName);

            if (project == null)
            {
                AddTeamProject(projectName);
                project = FindTeamProjectNode(projectName);
            }

            project.Add(new XElement(MAPPINGELEMENT,
                new XAttribute(MAPPINGVERSIONCONTROLATTRIBUTE, versionControlPath),
                new XAttribute(MAPPINGSUBSTDRIVELETTERATTRIBUTE, substDriveLetter),
                new XAttribute(MAPPINGNAMEATTRIBUTE, mappingName)));

            Save();
        }

        /// <summary>
        /// Removes a specific mapping from the configuration file. Changes are persisted to disk when function returns.
        /// </summary>
        /// <param name="projectName">Name of the Team Project</param>
        /// <param name="mappingName">Name of the mapping</param>
        public static void RemoveMappingFromProject(string projectName, string mappingName)
        {
           
                var teamProjectNode = from teamProject in ConfigXmlFile.Elements(TEAMPROJECTELEMENT)
                                      where teamProject.Attribute(TEAMPROJECTNAMEATTRIBUTE) != null
                                      && teamProject.Attribute(TEAMPROJECTNAMEATTRIBUTE).Value == projectName
                                      select teamProject;
                // now get the specific node
                var substMappingNode = (from substMapping in teamProjectNode.Elements()
                                           where (substMapping.Attribute(SubstConfigurationFile.MAPPINGNAMEATTRIBUTE) != null &&
                                                 mappingName.Contains(substMapping.Attribute(SubstConfigurationFile.MAPPINGNAMEATTRIBUTE).Value))
                                           select substMapping).FirstOrDefault<XElement>();
                             
            if(substMappingNode!= null)
            {
                substMappingNode.Remove();
                Save();
            }
            else
            {
                throw new ArgumentException(string.Format("Mapping '{0}' can not be found in the project collection '{1}'", mappingName, projectName));
            }
        }

        /// <summary>
        /// Gets the configuration information for a specific mapping
        /// </summary>
        /// <param name="projectName">Name of the Team Project</param>
        /// <param name="mappingName">Name of the mapping</param>
        /// <returns></returns>
        public static SubstConfigNode GetMappingByName(string projectName, string mappingName)
        {

            var mappings = GetMappingsForProject(projectName);

            SubstConfigNode mapping = (from substMapping in mappings
                                where (substMapping.name != null &&
                                      mappingName.Contains( substMapping.name ))
                                select substMapping).FirstOrDefault<SubstConfigNode>();
            return mapping;
        }

        /// <summary>
        /// Edits an existing mapping and changes the values to reflect the provided parameters. Changes are commited to file on disk when function returns.
        /// All parameters must be supplied. If no change of a certain parameter is required, just provide the previous value.
        /// </summary>
        /// <param name="projectName">Name of the Team Project the mapping is in</param>
        /// <param name="oldMappingName">Name of the mapping before the changes</param>
        /// <param name="mappingName">New Mapping name</param>
        /// <param name="versionControlPath">New version control path. This is in the server format $&lt;TeamprojectName&gt;\yourlocation.</param>
        /// <param name="substDriveLetter">Drive letter to use for the mapping. Only use one character, no colon postfix, etc. Just the letter of the drive only!</param>
        public static void ModifyMappingInProject(string projectName, string oldMappingName, string mappingName, string versionControlPath, string substDriveLetter)
        {
            RemoveMappingFromProject(projectName, oldMappingName);
            AddMappingToProject(projectName, mappingName, versionControlPath, substDriveLetter);
        }

        
        /// <summary>
        /// Get from the configuration file all mappings for a specific team project.
        /// </summary>
        /// <param name="projectName">Name of the Team Project you want the list of subst mappings for</param>
        /// <returns>IEnumerable&lt;SubstconfigNode&gt; containing the found mapping definitions</returns>
        public static IEnumerable<SubstConfigNode> GetMappingsForProject(string projectName)
        {
            
            var substNodes = (from mapping in ConfigXmlFile.Elements(TEAMPROJECTELEMENT)
                             where mapping.Attribute(TEAMPROJECTNAMEATTRIBUTE) != null
                             && mapping.Attribute(TEAMPROJECTNAMEATTRIBUTE).Value == projectName
                             select mapping).Descendants();

            return from substConfig in substNodes
                   select new SubstConfigNode
                   {
                     name = substConfig.Attribute(SubstConfigurationFile.MAPPINGNAMEATTRIBUTE).Value,
                     drive = substConfig.Attribute(SubstConfigurationFile.MAPPINGSUBSTDRIVELETTERATTRIBUTE).Value,
                     versionControlPath = substConfig.Attribute(SubstConfigurationFile.MAPPINGVERSIONCONTROLATTRIBUTE).Value
                    };
        }

        /// <summary>
        /// Returns the list of images we use as icons for the substexplorer. Images are writen to disk if they do not
        /// already exist. Images are embeded as resource of the assembly.
        /// </summary>
        /// <returns>ImageList containing 3 entries. entry 0 contains rootImage, entry 1 the not mapped entry and entry 2 the mapped image.</returns>
        public static ImageList GetCommandImages()
        {
            if (_imageList == null)
            {
                if (!Directory.Exists(SubstExplorerFolder))
                {
                    Directory.CreateDirectory(SubstExplorerFolder);
                    // now copy the files to the local directory
                    ResourceWriter.WriteEmbeddedResources(Assembly.GetExecutingAssembly(), "png", SubstExplorerFolder);
                }
                _imageList = new ImageList();
                _imageList.Images.Add(Image.FromFile(Path.Combine(SubstExplorerFolder, "SubstExplorerRoot.png")));
                _imageList.Images.Add(Image.FromFile(Path.Combine(SubstExplorerFolder, "SubstNotMapped.png")));
                _imageList.Images.Add(Image.FromFile(Path.Combine(SubstExplorerFolder, "SubstMapped.png")));
            }
            return _imageList;
        }
#endregion
    }
}
