﻿#region Copyright © 2009 Marcel de Vries
/* This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in a
 *    product, an acknowledgment in the product documentation is requested, as
 *    shown here:
 * 
 *    Portions copyright © 2009 Marcel de Vries (http://blogs.infosupport.com/blogs/marcelv).
 * 
 * 2. No substantial portion of this source code may be redistributed without
 *    the express written permission of the copyright holders, where
 *    "substantial" is defined as enough code to be recognizably from this code.
 */
#endregion
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using EnvDTE;
using Microsoft.TeamFoundation.Common;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.VisualStudio.TeamFoundation.VersionControl;
using Microsoft.Win32;
namespace MSDNMagazine.TFSPlugins
{
    public class SubstExplorerLeaf : BaseHierarchyNode
    {
        private static List<SubstExplorerLeaf> _substedNodes = new List<SubstExplorerLeaf>();

        private enum SubstIconId
        {
            unsubsted = 1,
            substed = 2
        }

        CommandID command = new CommandID(GuidList.guidPackageCmdSet, CommandList.mnuDelete);
        bool IsDriveSubsted { get; set; }

        public string VersionControlPath { get; set; }
        public string SubstDriveLetter { get; set; }

        public SubstExplorerLeaf(string path,  string substDriveLetter, string versionControlPath)
            : base(path, path + " (" + substDriveLetter + ":)")
        {
            this.ImageIndex = (int)SubstIconId.unsubsted;
            this.NodePriority = (int)TeamExplorerNodePriority.Leaf;
            
            this.VersionControlPath = versionControlPath;
            this.SubstDriveLetter = substDriveLetter;
            this.IsDriveSubsted = false;

            // Base class has two private members that hold the icon hanldes.
            // these are initialized as null pointers and when we first
            // add a node to the hierarchy this will result in stange behavior of the icons being displayed.
            // Now use reflection to set the icon handles to correct existing icons.
            SetCorrectIconHandles();
        }

        private void SetCorrectIconHandles()
        {
            // default icon is non substed drive... index 0;
            FieldInfo fieldInfo = typeof(BaseHierarchyNode).GetField("m_iconHandle", BindingFlags.FlattenHierarchy | BindingFlags.Instance | BindingFlags.NonPublic);
            fieldInfo.SetValue(this, new Bitmap(SubstConfigurationFile.GetCommandImages().Images[0]).GetHicon());

            fieldInfo = typeof(BaseHierarchyNode).GetField("m_openFoderIconHandle", BindingFlags.FlattenHierarchy | BindingFlags.Instance | BindingFlags.NonPublic);
            fieldInfo.SetValue(this, new Bitmap(SubstConfigurationFile.GetCommandImages().Images[0]).GetHicon());
        }

        public override void DoDefaultAction()
        {
            if (!IsDriveSubsted)
            {
                SubstDrive();
            }
            else
            {
                UnsubstDrive(SubstDriveLetter);
            }
        }

        public override CommandID ContextMenu
        {
            get
            {
                return command;
            }
        }

        public override string PropertiesClassName
        {
            get { return "Subst Leaf Node"; }
        }

        public override ICustomTypeDescriptor PropertiesObject
        {
            get
            {
                return new SubstExplorerProperties(this);
            }
        }


        private void SubstDrive()
        {
            if (IsDriveAlreadySubsted(SubstDriveLetter))
            {
                UnsubstDrive(SubstDriveLetter);
            }

            string substresponse = SubstHelper.Subst(SubstDriveLetter, GetLocalFolder());

            if (string.IsNullOrEmpty(substresponse))
            {
                IsDriveSubsted = true;
                _substedNodes.Add(this);
                this.ImageIndex = (int)SubstIconId.substed;
            }
            else
            {
                MessageBox.Show(string.Format("Unable to make subst mapping. Message:\n {0}", substresponse));
            }
        }

        private bool IsDriveAlreadySubsted(string driveLetter)
        {
            bool IsdrivePhysicalyMaped = SubstHelper.SubstedDrives().Where(d => d.Contains(driveLetter + ":\\")).Count() != 0;
            bool IsdriveKnownToBeMaped = (from substedNode in _substedNodes
                                          where substedNode.SubstDriveLetter == driveLetter
                                          select substedNode).ToArray<SubstExplorerLeaf>().Length > 0;

            return IsdriveKnownToBeMaped || IsdrivePhysicalyMaped;
        }


        private string GetLocalFolder()
        {
            VersionControlServer vcs = (VersionControlServer)((SubstExplorerUIHierarchy)ParentHierarchy).tfs.GetService(typeof(VersionControlServer));
            Workspace[] workspaces = vcs.QueryWorkspaces(null, vcs.AuthenticatedUser, Environment.MachineName);

            foreach (Workspace ws in workspaces)
            {
                WorkingFolder wf = ws.TryGetWorkingFolderForServerItem(VersionControlPath);
                if (wf != null)
                {
                    // We found a workspace that contains this versioncontrolled item
                    // get the local location so we can map the drive to this location....
                    return wf.LocalItem;
                }
            }
            return null;
        }

        private void UnsubstDrive(string driveLetter)
        {
            SubstExplorerLeaf[] nodesWithSameDriveSubst = (from substedNode in _substedNodes
                                                           where substedNode.SubstDriveLetter == driveLetter
                                                           select substedNode).ToArray<SubstExplorerLeaf>();

            if (nodesWithSameDriveSubst.Count() > 0)
            {
                foreach (var node in nodesWithSameDriveSubst)
                {
                    node.UnsubstNode();
                }
            }
            else
            {
                // not a drive we substed in the TeamExplorer, unsubst the drive and remove 
                string substResponse = SubstHelper.DeleteSubst(driveLetter);
            }
        }

        public void UnsubstNode()
        {
            string substResponse = SubstHelper.DeleteSubst(SubstDriveLetter);
            IsDriveSubsted = false;
            _substedNodes.Remove(this);
            this.ImageIndex = (int)SubstIconId.unsubsted;
        }


        public string localPath
        {
            get { return VersionControlPath; }

        }

        public Workspace CurrentActiveWorkspace
        {
            get
            {
                DTE dteObject = BasicHelper.GetService<DTE>();
                VersionControlExt vce;

                vce = dteObject.GetObject("Microsoft.VisualStudio.TeamFoundation.VersionControl.VersionControlExt") as VersionControlExt;
                if (vce != null)
                {
                    Workspace ws = vce.SolutionWorkspace;
                    if (ws != null)
                    {
                        return ws;
                    }
                    else
                    {
                        // try to get the workspace from the VC explorer that can be loaded
                        if (vce.Explorer.Workspace != null)
                        {
                            return vce.Explorer.Workspace;
                        }
                        else
                        {
                            // last thing we can do is determine workspace last openend inthe team explorer
                            RegistryKey key = Registry.CurrentUser.OpenSubKey(@"Software\Microsoft\VisualStudio\9.0\TeamFoundation\SourceControl\Explorer");
                            Guid keyForCurrentConnectedServer = ((SubstExplorerUIHierarchy)ParentHierarchy).tfs.InstanceId;
                            RegistryKey explorerSubKey = key.OpenSubKey(keyForCurrentConnectedServer.ToString());
                            string lastOpenWorkspaceName = explorerSubKey.GetValue("LastWorkspace").ToString();
                            if (!string.IsNullOrEmpty(lastOpenWorkspaceName))
                            {
                                VersionControlServer vcs = (VersionControlServer)((SubstExplorerUIHierarchy)ParentHierarchy).tfs.GetService(typeof(VersionControlServer));
                                Workspace[] workspaces = vcs.QueryWorkspaces(lastOpenWorkspaceName, vcs.AuthenticatedUser, Environment.MachineName);
                                if (workspaces.Length > 0)
                                {
                                    return workspaces[0];
                                }
                            }
                        }
                        return null;
                    }
                }
                else
                    return null;

            }
        }
    }

}
