﻿#region Copyright © 2009 Marcel de Vries
/* This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in a
 *    product, an acknowledgment in the product documentation is requested, as
 *    shown here:
 * 
 *    Portions copyright © 2009 Marcel de Vries (http://blogs.infosupport.com/blogs/marcelv).
 * 
 * 2. No substantial portion of this source code may be redistributed without
 *    the express written permission of the copyright holders, where
 *    "substantial" is defined as enough code to be recognizably from this code.
 */
#endregion
using System;
using System.Windows.Forms;
using Dialogs;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Common;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;
namespace MSDNMagazine.TFSPlugins
{
    public class SubstExplorerUIHierarchy : BaseUIHierarchy, IVsHierarchyDeleteHandler, IVsHierarchyDeleteHandler2
    {
        public static TeamFoundationServer _tfs;
        public SubstExplorerUIHierarchy(IVsUIHierarchy parentHierarchy, uint itemId, BasicAsyncPlugin plugin)
            : base(parentHierarchy, itemId, plugin, MSDNMagazine.TFSPlugins.SubstExplorerHostPackage.Instance)
        {
        }
        
        public override int ExecCommand(uint itemId, ref Guid guidCmdGroup, uint nCmdId, uint nCmdExecOpt, IntPtr pvain, IntPtr p)
        {
            if (guidCmdGroup == GuidList.guidPackageCmdSet)
            {
                switch (nCmdId)
                {
                    case (uint)CommandList.cmdAdd:
                        AddNewDefinition(this.ProjectName);
                        return VSConstants.S_OK;
                    case (uint)CommandList.cmdDelete:
                        RemoveDefinition(itemId);
                        return VSConstants.S_OK;
                    case (uint)CommandList.cmdEdit:
                        EditDefinition(itemId);
                        return VSConstants.S_OK;
                    default: return VSConstants.E_FAIL;
                }
            }
            return base.ExecCommand(itemId, ref guidCmdGroup, nCmdId, nCmdExecOpt, pvain, p);
        }

        internal TeamFoundationServer tfs
        {
            get 
            {
                if ((_tfs != null) && (_tfs.Uri.Host != new Uri(ServerUrl).Host))
                {
                    // we reconected to a different host, reinitialize...
                    _tfs = null;
                }

                if (_tfs == null)
                {
                    //Creating a new instance will result in re-authenticating, so use the factory!
                    _tfs = TeamFoundationServerFactory.GetServer(ServerUrl, new UICredentialsProvider());
                    _tfs.Authenticate();
                }
                return _tfs;
            }
        }
        
        internal void AddNewDefinition(string projectName)
        {
            // reinitialize tfs if we connected to different host
            VersionControlServer vcs = (VersionControlServer)tfs.GetService(typeof(VersionControlServer));

            AddNewSubstDefDialog dialog = new AddNewSubstDefDialog(vcs, projectName);
            if (dialog.ShowDialog() != DialogResult.Cancel)
            {
                // save configuration
                SubstConfigurationFile.AddMappingToProject(projectName, dialog.substName, dialog.VersionControlPath, dialog.SubstDriveLetter);
                HierarchyNode.AddChild(new SubstExplorerLeaf(dialog.substName, dialog.SubstDriveLetter, dialog.VersionControlPath));
            }
        }
        internal void EditDefinition(uint itemId)
        {
            BaseHierarchyNode selectedNode = this.NodeFromItemId(itemId);
            // reinitialize tfs if we connected to different host
            VersionControlServer vcs = (VersionControlServer)tfs.GetService(typeof(VersionControlServer));

            AddNewSubstDefDialog dialog = new AddNewSubstDefDialog(vcs, this.ProjectName, selectedNode.Name);
            if (dialog.ShowDialog() != DialogResult.Cancel)
            {
                // save configuration
                SubstConfigurationFile.ModifyMappingInProject(this.ProjectName, selectedNode.Name, dialog.substName, dialog.VersionControlPath, dialog.SubstDriveLetter);
                RefreshTree(this.ProjectName);
            }
        }

        internal void RemoveDefinition(uint itemId)
        {
            BaseHierarchyNode selectedNode = this.NodeFromItemId(itemId);
                        
            if(AreYouSureToDelete(selectedNode.Name))
            {
                // first remove it from storage
                RemoveDefinitionFromFile(selectedNode);
                // remove it from the UI
                selectedNode.Remove();
            }
        }
        
        internal static void RemoveDefinitionFromFile(BaseHierarchyNode selectedNode)
        {
            SubstConfigurationFile.RemoveMappingFromProject(selectedNode.ProjectName, selectedNode.Name);
        }
        
        internal bool AreYouSureToDelete(string mappingName)
        {
            return MessageBox.Show(string.Format("Are you sure you want to remove the mapping '{0}'?", mappingName), "Remove mapping", MessageBoxButtons.YesNo) != DialogResult.No;
        }
        
        internal void RefreshTree(string projectName)
        {
            // Recreate the tree by clearing the tree first
            while (HierarchyNode.FirstChild != null)
            {
                HierarchyNode.FirstChild.Remove();
            }
            // repopulate
            SubstExplorer.PopulateTree(HierarchyNode);
        }

        #region IVsHierarchyDeleteHandler2 Members

        public int ShowMultiSelDeleteOrRemoveMessage(uint dwDelItemOp, uint cDelItems, uint[] rgDelItems, out int pfCancelOperation)
        {
            pfCancelOperation = Convert.ToInt32(true);
            return VSConstants.S_OK;
        }

        public int ShowSpecificDeleteRemoveMessage(uint dwDelItemOps, uint cDelItems, uint[] rgDelItems, out int pfShowStandardMessage, out uint pdwDelItemOp)
        {
            SubstExplorerLeaf nodeToDelete = NodeFromItemId(rgDelItems[0]) as SubstExplorerLeaf;
            if (AreYouSureToDelete(nodeToDelete.Name))
            {
                pdwDelItemOp = 1; // == DELITEMOP_DeleteFromStorage; DELITEMOP_RemoveFromProject==2; 
            }
            else 
            {
                pdwDelItemOp = 0; // NO delete, user selected NO option
            }
            pfShowStandardMessage = Convert.ToInt32(false);
            return VSConstants.S_OK;
        }

        #endregion

        #region IVsHierarchyDeleteHandler Members

        public int DeleteItem(uint dwDelItemOp, uint itemid)
        {
            SubstExplorerLeaf nodeToDelete = NodeFromItemId(itemid) as SubstExplorerLeaf;
            if (nodeToDelete != null)
            {
                // remove from storage
                RemoveDefinitionFromFile(nodeToDelete);
                // remove from UI
                nodeToDelete.Remove();
            }
            return VSConstants.S_OK;
        }

        public int QueryDeleteItem(uint dwDelItemOp, uint itemid, out int pfCanDelete)
        {
            pfCanDelete = Convert.ToInt32(NodeFromItemId(itemid) is SubstExplorerLeaf);
            return VSConstants.S_OK;
        }

        #endregion
    }
}


